/* RoomCamacAccess.cc */
/* Edited by Sanshiro E. on 6 October 1997. */
/* Last updated by Sanshiro E. on 18 June 2000. */


#include <strstream.h>
#include "RoomCamacAccess.hh"


TCamacController::TCamacController(void)
{
}

TCamacController::~TCamacController()
{
}

void TCamacController::Close(void)
{
}

TCamacCrate::TCamacCrate(void)
{
    _CamacController = 0;
}

TCamacCrate::TCamacCrate(TCamacController *Controller)
{
    _CamacController = Controller;
}

TCamacCrate::~TCamacCrate()
{
}

void TCamacCrate::InstallController(TCamacController *Controller)
{
    _CamacController = Controller;    
    _CamacController->Open();
}

void TCamacCrate::Install(TCamacModule *CamacModule, int StationNumber) throw(THardwareException)
{
    if (_CamacController == 0) {
        throw THardwareException(
            "TCamacCrate::Install()",
            "controller is not installed"
        );
    }

    CamacModule->Attach(this, StationNumber);
    _ModuleList.push_back(CamacModule);
}

void TCamacCrate::Uninstall(TCamacModule *CamacModule)
{
    CamacModule->Detach();
}

TCamacController* TCamacCrate::CamacController(void) const
{
    return _CamacController;
}


TCamacModule::TCamacModule(void)
{
    _CamacController = 0;
    _CamacCrate = 0;
    _StationNumber = 0;
    _StationNumberBit = 0;

    _IsLamEnabled = 0;
}

TCamacModule::~TCamacModule()
{
    if (_CamacCrate != 0) {
	_CamacCrate->Uninstall(this);
    }
}

TCamacModule* TCamacModule::Clone(void)
{
    return new TCamacModule();
}

int TCamacModule::EnableLam(void) throw(THardwareException)
{
    if (_IsLamEnabled) {
	return 0;
    }

    int Data;
    int Q, X;
    int Status = _CamacController->CamacAction(
        _StationNumber, adLamEnable, fnLamEnable, Data, Q, X
    );

/* Some of Camac modules don't support F26. */
/* In such case, No-X will be returned. */
#if 0
    if (!X) {
	throw THardwareException(
            "TCamacModule::EnableLam()", 
            "LAM enable fault (NoX)"
        );
    }
    if (!Q) {
	throw THardwareException(
            "TCamacModule::EnableLam()", 
            "LAM enable fault (NoQ)"
        );
    }
#endif

    _IsLamEnabled = 1;

    return Status;
}

int TCamacModule::DisableLam(void) throw(THardwareException)
{
    if (! _IsLamEnabled) {
	return 0;
    }

    int Data;
    int Q, X;
    int Status = _CamacController->CamacAction(
        _StationNumber, adLamDisable, fnLamDisable, Data, Q, X
    );

/* Some of Camac modules don't support F24. */
/* In such case, No-X will be returned. */
#if 0
    if (!X) {
	throw THardwareException(
            "TCamacModule::DisableLam()", 
            "LAM disable fault (NoX)"
        );
    }
    if (!Q) {
	throw THardwareException(
            "TCamacModule::DisableLam()", 
            "LAM disable fault (NoQ)"
        );
    }
#endif

    _IsLamEnabled = 0;

    return Status;
}

int TCamacModule::WaitLam(int TimeOut /* sec */) throw(THardwareException)
{
    if (! _IsLamEnabled) {
	EnableLam();
    }

    int Result = _CamacController->WaitLam(TimeOut);
    if (Result < 0) {
        // timed out
        return 0;
    }
    else {
        return (Result & Bit(_StationNumber - 1)) ? 1 : 0;
    }
}

bool TCamacModule::IsLamRequested(void)
{
/* KEKCC driver doesn't support ReadLam(). */
#if 0   
        return _CamacController->ReadLam() & _StationNumberBit;
#else
	int Data;
	int Q, X;
	_CamacController->CamacAction(_StationNumber, adLamTest, fnLamTest, Data, Q, X);

	return Q;
#endif
}

int TCamacModule::StationNumber(void)
{
    return _StationNumber;
}

int TCamacModule::CamacAction(int Address, int Function, int &Data, int &Q, int &X) 
{
    _CamacController->CamacAction(_StationNumber, Address, Function, Data, Q, X);

    return Q;
}

int TCamacModule::CamacAction(int Address, int Function, int &Data) throw(THardwareException)
{
    int Q, X;

    _CamacController->CamacAction(_StationNumber, Address, Function, Data, Q, X);
    if (! X) {
	ostrstream Message;
	Message << "No X-response at station " << _StationNumber << "." << ends;
	throw THardwareException("TCamacModule::CamacAction", Message.str());
    }

    return Q;
}

int TCamacModule::CamacAction(int Address, int Function) throw(THardwareException)
{
    int Data;
    int Q, X;

    _CamacController->CamacAction(_StationNumber, Address, Function, Data, Q, X);
    if (! X) {
	ostrstream Message;
	Message << "No X-response at station " << _StationNumber << "." << ends;
	throw THardwareException("TCamacModule::CamacAction", Message.str());
    }

    return Q;
}

void TCamacModule::Attach(TCamacCrate* Crate, int StationNumber)
{
    _CamacCrate = Crate;
    _CamacController = Crate->CamacController();
    _StationNumber = StationNumber;
    _StationNumberBit = Bit(_StationNumber - 1);
}

void TCamacModule::Detach()
{
    _CamacCrate = 0;
    _CamacController = 0;
    _StationNumber = 0;
    _StationNumberBit = 0;
}

int TCamacModule::EnableInterrupt(void) throw(THardwareException)
{
    return EnableLam();
}

int TCamacModule::DisableInterrupt(void) throw(THardwareException)
{
    return DisableLam();
}
