/* RoomCamacAccess.hh */
/* Edited by Sanshiro E. on 23 October 1997. */
/* Last updated by Sanshiro E. on 18 June 2000. */

#ifndef __RoomCamacAccess_hh__
#define __RoomCamacAccess_hh__


#include <vector>
#include "RoomDefs.hh"
#include "RoomDeviceBase.hh"
#include "RoomModule.hh"

class TCamacModule;


class TCamacController: public TController {
  public:
    TCamacController(void);
    virtual ~TCamacController();
    virtual TCamacController* Clone(void) = 0;
  public:
    virtual void Open(void) throw(THardwareException) = 0;
    virtual void Close(void);
    virtual int CamacAction(int StationNumber, int Address, int Function, int &Data, int &Q, int &X) throw(THardwareException) = 0;
    virtual int Initialize(void) throw(THardwareException) = 0;
    virtual int Clear(void) throw(THardwareException) = 0;
    virtual int Inhibit(void) throw(THardwareException) = 0;
    virtual int ReleaseInhibit(void) throw(THardwareException) = 0;
    virtual int EnableInterruptNotification(void) throw(THardwareException) = 0;
    virtual int DisableInterruptNotification(void) throw(THardwareException) = 0;
    virtual int ReadLam(void) throw(THardwareException) = 0;
    virtual int WaitLam(int TimeOut /* sec */) throw(THardwareException) = 0;
};


class TCamacCrate: public TCrate {
  public:
    TCamacCrate(void);
    TCamacCrate(TCamacController *Controller);
    virtual ~TCamacCrate();
    virtual void InstallController(TCamacController *Controller);
    virtual void Install(TCamacModule *CamacModule, int StationNumber) throw(THardwareException);
    virtual void Uninstall(TCamacModule *CamacModule);
    virtual TCamacController* CamacController(void) const;
  private:
    TCamacController *_CamacController;
    vector<TCamacModule*> _ModuleList;
};


class TCamacModule: public TModule {
  public:
    TCamacModule(void);
    virtual ~TCamacModule();
    virtual TCamacModule* Clone(void);
    virtual int EnableLam(void) throw(THardwareException);
    virtual int DisableLam(void) throw(THardwareException);
    virtual int WaitLam(int TimeOut /* sec */) throw(THardwareException);
    virtual bool IsLamRequested(void);
    int StationNumber(void);
    int CamacAction(int Address, int Function, int &Data, int &Q, int &X);
    int CamacAction(int Address, int Function, int &Data) throw(THardwareException);
    int CamacAction(int Address, int Function) throw(THardwareException);
    virtual int EnableInterrupt(void) throw(THardwareException);
    virtual int DisableInterrupt(void) throw(THardwareException);
  private:
    friend class TCamacCrate;
    void Attach(TCamacCrate* Crate, int StationNumber);
    void Detach(void);
  protected:
    enum TFunctionTable {
	fnLamTest = 8,
	fnLamEnable = 26, fnLamDisable = 24
    };
    enum TAddressTable {
	adLamTest = 0,
	adLamEnable = 0, adLamDisable = 0
    };
    TCamacController *_CamacController;
    TCamacCrate *_CamacCrate;
    int _StationNumber;
    unsigned _StationNumberBit;
    bool _IsLamEnabled;
};

#endif
