/* DeviceFactory.cc */
/* Edited by Sanshiro E. on 7 May 1999. */
/* Last updated by Sanshiro E. on 18 June 2000. */


#include <string>
#include <map>
#include "RoomModule.hh"
#include "RoomVmeAccess.hh"
#include "RoomCamacAccess.hh"
#include "RoomSoftwareDevice.hh"
#include "RoomDeviceFactory.hh"


TDeviceFactory* TDeviceFactory::_Instance = 0;

TDeviceFactory::TDeviceFactory()
{
}

TDeviceFactory::~TDeviceFactory()
{
}

TDeviceFactory* TDeviceFactory::GetInstance(void)
{
    if (_Instance == 0) {
        _Instance = new TDeviceFactory();
    }

    return _Instance;
}

string TDeviceFactory::ReducedNameOf(string Name) const
{
    // replace '-' or ' ' with '_'
    for (unsigned i = 0; i < Name.size(); i++) {
	if ((Name[i] == '-') || (Name[i] == ' ')) {
	    Name[i] = '_';
	}
    }

    // if Name contains more than three words, remove the first word.
    if (Name.find('_') != Name.rfind('_')) {
	return Name.substr(Name.find('_') + 1);
    }
    else {
	return Name;
    }
}


void TDeviceFactory::RegisterVmeController(const string& Name, TVmeController* Controller)
{
    _VmeControllerTable[Name] = Controller;
}

void TDeviceFactory::RegisterCamacController(const string& Name, TCamacController* Controller)
{
    _CamacControllerTable[Name] = Controller;
}

void TDeviceFactory::RegisterVmeModule(const string& Name, TVmeModule* Module)
{
    _VmeModuleTable[Name] = Module;
}

void TDeviceFactory::RegisterCamacModule(const string& Name, TCamacModule* Module)
{
    _CamacModuleTable[Name] = Module;
}

void TDeviceFactory::RegisterSoftwareModule(const string& Name, TSoftwareModule* Module)
{
    _SoftwareModuleTable[Name] = Module;
}

TVmeController* TDeviceFactory::CreateVmeController(const string& Name) throw(THardwareException)
{
    TVmeController* ControllerPrototype;

    if (Name.empty()) {
	if (! _VmeControllerTable.empty()) {
	    ControllerPrototype = (*_VmeControllerTable.begin()).second;
	}
	else {
	    throw THardwareException(
		"TDeviceFactory::CreateVmeController()",
		"No VME controller is registered."
	    );
	}
    }
    else {
	string ReducedName = ReducedNameOf(Name);
	if (_VmeControllerTable.count(ReducedName) > 0) {
	    ControllerPrototype = _VmeControllerTable[ReducedName];
	}
	else {
	    throw THardwareException(
		"TDeviceFactory::CreateVmeController()",
		"No such VME controller: " + Name
	    );
	}
    }

    return ControllerPrototype->Clone();
}

TCamacController* TDeviceFactory::CreateCamacController(const string& Name) throw(THardwareException)
{
    TCamacController* ControllerPrototype = 0;

    if (Name.empty()) {
	if (! _CamacControllerTable.empty()) {
	    ControllerPrototype = (*_CamacControllerTable.begin()).second;
	}
	else {
	    throw THardwareException(
		"TDeviceFactory::CreateCamacController()",
		"No CAMAC controller is registered."
	    );
	}
    }
    else {
	string ReducedName = ReducedNameOf(Name);
	if (_CamacControllerTable.count(ReducedName) > 0) {
	    ControllerPrototype = _CamacControllerTable[ReducedName];
	}
	else {
	    throw THardwareException(
		"TDeviceFactory::CreateCamacController()",
		"No such CAMAC controller: " + Name
	    );
	}
    }

    return ControllerPrototype->Clone();
}

TVmeCrate* TDeviceFactory::CreateVmeCrate(const string& Name) throw(THardwareException)
{
    return new TVmeCrate();
}

TCamacCrate* TDeviceFactory::CreateCamacCrate(const string& Name) throw(THardwareException)
{
    return new TCamacCrate();
}

TVmeModule* TDeviceFactory::CreateVmeModule(const string& Name) throw(THardwareException)
{
    string ReducedName = ReducedNameOf(Name);

    if (_VmeModuleTable.count(ReducedName) == 0) {
        throw THardwareException(
            "TDeviceFactory::CreateVmeModule()",
            "No such software module: " + Name
        );
    }

    return _VmeModuleTable[ReducedName]->Clone();
}

TCamacModule* TDeviceFactory::CreateCamacModule(const string& Name) throw(THardwareException)
{
    string ReducedName = ReducedNameOf(Name);

    if (_CamacModuleTable.count(ReducedName) == 0) {
        throw THardwareException(
            "TDeviceFactory::CreateCamacModule()",
            "No such software module: " + Name
        );
    }

    return _CamacModuleTable[ReducedName]->Clone();
}

TSoftwareModule* TDeviceFactory::CreateSoftwareModule(const string& Name) throw(THardwareException)
{
    if (_SoftwareModuleTable.count(Name) == 0) {
        throw THardwareException(
            "TDeviceFactory::CreateSoftwareModule()",
            "No such software module: " + Name
        );
    }

    return _SoftwareModuleTable[Name]->Clone();
}



TVmeControllerCreator::TVmeControllerCreator(const string& Name, TVmeController* Controller)
{
    TDeviceFactory::GetInstance()->RegisterVmeController(Name, Controller);
}

TCamacControllerCreator::TCamacControllerCreator(const string& Name, TCamacController* Controller)
{
    TDeviceFactory::GetInstance()->RegisterCamacController(Name, Controller);
}

TVmeModuleCreator::TVmeModuleCreator(const string& Name, TVmeModule* Module)
{
    TDeviceFactory::GetInstance()->RegisterVmeModule(Name, Module);
}

TCamacModuleCreator::TCamacModuleCreator(const string& Name, TCamacModule* Module)
{
    TDeviceFactory::GetInstance()->RegisterCamacModule(Name, Module);
}

TSoftwareModuleCreator::TSoftwareModuleCreator(const string& Name, TSoftwareModule* Module)
{
    TDeviceFactory::GetInstance()->RegisterSoftwareModule(Name, Module);
}
