/* RoomDeviceFactory.hh */
/* Edited by Sanshiro E. on 7 May 1999. */
/* Last updated by Sanshiro E. on 18 June 2000. */

#ifndef __RoomDeviceFactory_hh__
#define __RoomDeviceFactory_hh__


#include <string>
#include <map>
#include "RoomDefs.hh"
#include "RoomDeviceBase.hh"
#include "RoomModule.hh"
#include "RoomVmeAccess.hh"
#include "RoomCamacAccess.hh"
#include "RoomSoftwareDevice.hh"


class TDeviceFactory {
  public:
    static TDeviceFactory* GetInstance(void);
    virtual ~TDeviceFactory();
    virtual void RegisterVmeController(const string& Name, TVmeController* Controller);
    virtual void RegisterCamacController(const string& Name, TCamacController* Controller);
    virtual void RegisterVmeModule(const string& Name, TVmeModule* Module);
    virtual void RegisterCamacModule(const string& Name, TCamacModule* Module);
    virtual void RegisterSoftwareModule(const string& Name, TSoftwareModule* Module);
    virtual TVmeModule* CreateVmeModule(const string& Name) throw(THardwareException);
    virtual TCamacModule* CreateCamacModule(const string& Name) throw(THardwareException);
    virtual TSoftwareModule* CreateSoftwareModule(const string& Name) throw(THardwareException);
    virtual TVmeCrate* CreateVmeCrate(const string& Name = "") throw(THardwareException);
    virtual TCamacCrate* CreateCamacCrate(const string& Name = "") throw(THardwareException);
    virtual TVmeController* CreateVmeController(const string& Name = "") throw(THardwareException);
    virtual TCamacController* CreateCamacController(const string& Name = "") throw(THardwareException);
  protected:
    TDeviceFactory(void);
    string ReducedNameOf(string Name) const;
  protected:
    static TDeviceFactory* _Instance;
    map<string, TVmeController*, less<string> > _VmeControllerTable;
    map<string, TCamacController*, less<string> > _CamacControllerTable;
    map<string, TVmeModule*, less<string> > _VmeModuleTable;
    map<string, TCamacModule*, less<string> > _CamacModuleTable;
    map<string, TSoftwareModule*, less<string> > _SoftwareModuleTable;
};



class TVmeControllerCreator {
  public:
    TVmeControllerCreator(const string& Name, TVmeController* Controller);
};

class TCamacControllerCreator {
  public:
    TCamacControllerCreator(const string& Name, TCamacController* Controller);
};

class TVmeModuleCreator {
  public:
    TVmeModuleCreator(const string& Name, TVmeModule* Module);
};

class TCamacModuleCreator {
  public:
    TCamacModuleCreator(const string& Name, TCamacModule* Module);
};

class TSoftwareModuleCreator {
  public:
    TSoftwareModuleCreator(const string& Name, TSoftwareModule* Module);
};


#endif
