/* controller-Kinoko_Vmedrv.cc */
/* Edited by Sanshiro E. on 11 December 1999. */
/* Last updated by G H-S. on 18 July 2000.*/


#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <sys/mman.h>
#include <sys/ioctl.h>
#include "vmedrv.h"
#include "RoomDeviceFactory.hh"
#include "controller-Kinoko_Vmedrv.hh"


static TVmeControllerCreator Creator1(
    "Kinoko_Vmedrv", new TVmeController_Kinoko_Vmedrv()
);
static TVmeControllerCreator Creator2(
    "Bit3_617", new TVmeController_Kinoko_Vmedrv()
);
static TVmeControllerCreator Creator3(
    "SBS_617", new TVmeController_Kinoko_Vmedrv()
);


static const char *VmedrvDeviceFileName[] = {
    /* this must be consistent with TVmeTarnsferMode */
    /* (declared in ParaVmeAccess.hh) */
    "/dev/vmedrv16d16", "/dev/vmedrv16d32",
    "/dev/vmedrv24d16", "/dev/vmedrv24d32", 
    "/dev/vmedrv32d16", "/dev/vmedrv32d32"
};

static const char *VmedrvDmaDeviceFileName[] = {
    /* this must be consistent with TVmeTarnsferMode */
    /* (declared in ParaVmeAccess.hh) */
    "/dev/vmedrv16d16", "/dev/vmedrv16d32",
    "/dev/vmedrv24d16dma", "/dev/vmedrv24d32dma", 
    "/dev/vmedrv32d16dma", "/dev/vmedrv32d32dma"
};


TVmeController_Kinoko_Vmedrv::TVmeController_Kinoko_Vmedrv(void)
{
}

TVmeController_Kinoko_Vmedrv::~TVmeController_Kinoko_Vmedrv()
{
}

TVmeController* TVmeController_Kinoko_Vmedrv::Clone(void)
{
    return new TVmeController_Kinoko_Vmedrv();
}

const char* TVmeController_Kinoko_Vmedrv::DeviceFileName(TVmeTransferMode TransferMode)
{
    return VmedrvDeviceFileName[TransferMode];
}

const char* TVmeController_Kinoko_Vmedrv::DmaDeviceFileName(TVmeTransferMode TransferMode)
{
    return VmedrvDmaDeviceFileName[TransferMode];
}

void TVmeController_Kinoko_Vmedrv::RegisterInterruptNotification(TVmeAccessProperty* Property, int InterruptNumber, int Vector, int SignalId) throw(THardwareException)
{
    TVmeController::RegisterInterruptNotification(
	Property, InterruptNumber, Vector, SignalId
    );

    struct vmedrv_interrupt_property_t InterruptProperty;
    InterruptProperty.irq = Property->InterruptNumber;
    InterruptProperty.vector = Property->InterruptVector;
    InterruptProperty.signal_id = SignalId;

    int Status = ioctl(
	Property->DeviceDescriptor, 
	VMEDRV_IOC_REGISTER_INTERRUPT, 
	&InterruptProperty
    );
    if (Status < 0) {
	throw THardwareException(
            "TVmeController_Kinoko_Vmedrv::RegisterInterruptNotification(): "
	    "ioctl(2)",
	    strerror(errno)
        );
    }
}

void TVmeController_Kinoko_Vmedrv::EnableInterruptNotification(TVmeAccessProperty* Property) throw(THardwareException)
{
    struct vmedrv_interrupt_property_t InterruptProperty;
    InterruptProperty.irq = Property->InterruptNumber;
    InterruptProperty.vector = Property->InterruptVector;
    InterruptProperty.signal_id = Property->InterruptSignalId;

    int Status = ioctl(
	Property->DeviceDescriptor, 
	VMEDRV_IOC_ENABLE_INTERRUPT
    );
    if (Status < 0) {
	throw THardwareException(
            "TVmeController_Kinoko_Vmedrv::EnableInterruptNotification(): "
	    "ioctl(2)",
	    strerror(errno)
        );
    }
}

void TVmeController_Kinoko_Vmedrv::DisableInterruptNotification(TVmeAccessProperty* Property) throw(THardwareException)
{
    struct vmedrv_interrupt_property_t InterruptProperty;
    InterruptProperty.irq = Property->InterruptNumber;
    InterruptProperty.vector = Property->InterruptVector;
    InterruptProperty.signal_id = Property->InterruptSignalId;

    int Status = ioctl(
	Property->DeviceDescriptor, 
	VMEDRV_IOC_UNREGISTER_INTERRUPT, 
	&InterruptProperty
    );
    if (Status < 0) {
	throw THardwareException(
            "TVmeController_Kinoko_Vmedrv::DisableInterruptNotification(): "
	    "ioctl(2)",
	    strerror(errno)
        );
    }
}

int TVmeController_Kinoko_Vmedrv::WaitForInterruptNotification(TVmeAccessProperty* Property, unsigned TimeOut) throw(THardwareException)
{
    struct vmedrv_interrupt_property_t InterruptProperty;
    InterruptProperty.irq = Property->InterruptNumber;
    InterruptProperty.vector = Property->InterruptVector;
    InterruptProperty.signal_id = 0;
    InterruptProperty.timeout = TimeOut;

    int Status = ioctl(
	Property->DeviceDescriptor, 
	VMEDRV_IOC_WAIT_FOR_INTERRUPT, 
	&InterruptProperty
    );
    if ((Status < 0) && (errno == ENODEV)) {
	// this error may be due to interrupt never registered -- try.
	Status= ioctl(
	    Property->DeviceDescriptor, 
	    VMEDRV_IOC_REGISTER_INTERRUPT, 
	    &InterruptProperty
	);
	if (Status >= 0) {
	    Status = ioctl(
		Property->DeviceDescriptor, 
		VMEDRV_IOC_WAIT_FOR_INTERRUPT, 
		&InterruptProperty
	    );
	}
    }
    else if ((Status < 0) && (errno != ETIMEDOUT)) {
	throw THardwareException(
            "TVmeController_Kinoko_Vmedrv::WaitForInterrupt(): "
	    "ioctl(2)",
	    strerror(errno)
	);
    }
    
    return Status;
}
