/* module-CamacModules.cc */
/* Edited by Sanshiro E. on 26 May 1997. */
/* Last updated by Sanshiro E. on 28 October 2000. */


#include "RoomDeviceFactory.hh"
#include "RoomModule.hh"
#include "RoomCamacAccess.hh"
#include "module-CamacModules.hh"


static TCamacModuleCreator Creator0(
    "Standard", new TStandardCamacModule()
);
static TCamacModuleCreator Creator1(
    "Rinei_RPC022", new TCamacQADC_Rinei_RPC022()
);
static TCamacModuleCreator Creator2(
    "Hoshin_C009", new TCamacQADC_Hoshin_C009()
);
static TCamacModuleCreator Creator3(
    "Rinei_RPC060", new TCamacTDC_Rinei_RPC060()
);
static TCamacModuleCreator Creator4(
    "Hoshin_C006", new TCamacTDC_Hoshin_C006()
);
static TCamacModuleCreator Creator5(
    "Rinei_PRC081", new TCamacFADC_Rinei_RPC081()
);



TStandardCamacModule::TStandardCamacModule(void)
{
}

TStandardCamacModule::~TStandardCamacModule()
{
}

TCamacModule* TStandardCamacModule::Clone(void)
{
    return new TStandardCamacModule();
}

int TStandardCamacModule::WriteRegister(int Address, int Data) throw(THardwareException)
{
    static const int WriteFunction = 16;
    return CamacAction(Address, WriteFunction, Data);
}

int TStandardCamacModule::ReadRegister(int Address, int& Data) throw(THardwareException)
{
    static const int ReadFunction = 0;
    return CamacAction(Address, ReadFunction, Data);
}

int TStandardCamacModule::HasData(int Channel = -1) throw(THardwareException)
{
    return IsLamRequested();
}

int TStandardCamacModule::WaitData(unsigned TimeOut = 3600 /* sec */) throw(THardwareException)
{
    return WaitLam(TimeOut);
}

int TStandardCamacModule::Read(int Channel, int &Data) throw(THardwareException)
{
    static const int ReadFunction = 0;
    return CamacAction(Channel, ReadFunction, Data);
}

int TStandardCamacModule::Clear(int Channel) throw(THardwareException)
{
    static const int ClearFunction = 9;

    if (Channel < 0) {
	Channel = 0;
    }
    return CamacAction(Channel, ClearFunction);
}



TCommonplaceCamacModule::TCommonplaceCamacModule(TCamacParameters& CamacParameters)
: _CamacParameters(CamacParameters)
{
}

TCommonplaceCamacModule::TCommonplaceCamacModule(void)
{
}

TCommonplaceCamacModule::~TCommonplaceCamacModule() 
{
    DisableLam();
}

TCamacModule* TCommonplaceCamacModule::Clone(void)
{
    return new TCommonplaceCamacModule(_CamacParameters);
}

int TCommonplaceCamacModule::Read(int Channel, int &Data) throw(THardwareException)
{
    return CamacAction(Channel, _CamacParameters.ReadFunction, Data);
}

int TCommonplaceCamacModule::Clear(int Channel) throw(THardwareException)
{
    if (Channel < 0) {
	Channel = 0;
    }
    return CamacAction(Channel, _CamacParameters.ClearFunction);
}

int TCommonplaceCamacModule::HasData(int Channel) throw(THardwareException)
{
    return IsLamRequested();
}

int TCommonplaceCamacModule::WaitData(unsigned TimeOut) throw(THardwareException)
{
    return WaitLam(TimeOut);
}

int TCommonplaceCamacModule::NumberOfChannels(void) throw(THardwareException)
{
    return _CamacParameters.NumberOfChannels;
}

int TCommonplaceCamacModule::NumberOfSamples(void) throw(THardwareException)
{
    return _CamacParameters.NumberOfSamples;
}


TCamacQADC_Rinei_RPC022::TCamacQADC_Rinei_RPC022(void)
{
    _CamacParameters.NumberOfChannels = _NumberOfChannels;
    _CamacParameters.NumberOfSamples = _NumberOfSamples;
    _CamacParameters.ReadFunction = fnDataRead;
    _CamacParameters.ClearFunction = fnClear;
}

TCamacQADC_Hoshin_C009::TCamacQADC_Hoshin_C009(void)
{
    _CamacParameters.NumberOfChannels = _NumberOfChannels;
    _CamacParameters.NumberOfSamples = _NumberOfSamples;
    _CamacParameters.ReadFunction = fnDataRead;
    _CamacParameters.ClearFunction = fnClear;
}

TCamacTDC_Rinei_RPC060::TCamacTDC_Rinei_RPC060(void)
{
    _CamacParameters.NumberOfChannels = _NumberOfChannels;
    _CamacParameters.NumberOfSamples = _NumberOfSamples;
    _CamacParameters.ReadFunction = fnDataRead;
    _CamacParameters.ClearFunction = fnClear;
}

TCamacTDC_Hoshin_C006::TCamacTDC_Hoshin_C006(void)
{
    _CamacParameters.NumberOfChannels = _NumberOfChannels;
    _CamacParameters.NumberOfSamples = _NumberOfSamples;
    _CamacParameters.ReadFunction = fnDataRead;
    _CamacParameters.ClearFunction = fnClear;
}

TCamacFADC_Rinei_RPC081::TCamacFADC_Rinei_RPC081(void)
{
    _CamacParameters.NumberOfChannels = _NumberOfChannels;
    _CamacParameters.NumberOfSamples = _NumberOfSamples;
    _CamacParameters.ReadFunction = fnDataRead;
    _CamacParameters.ClearFunction = fnClear;
}

TCamacModule* TCamacFADC_Rinei_RPC081::Clone(void)
{
    return new TCamacFADC_Rinei_RPC081();
}

int TCamacFADC_Rinei_RPC081::Clear(int Channel) throw(THardwareException)
{
    if (Channel < 0) {
	for (Channel = 0; Channel < _NumberOfChannels; Channel++) {
	    CamacAction(Channel, fnClear);
	}
    }
    else {
	CamacAction(Channel, fnClear);
    }

    return 0;
}
