/* module-SIS_36xx.cc */
/* Created by Glenn H-S. on 31 Aug 2000. */
/* Last updated by Glenn H-S. on 31 Aug 2000. */

#include "RoomDeviceFactory.hh"
#include "RoomVmeAccess.hh"
#include "module-SIS_36xx.hh"


static TVmeModuleCreator Creator("SIS_36xx", new TVmeLatch_SIS_36xx());


TVmeLatch_SIS_36xx::TVmeLatch_SIS_36xx()
  :TVmeModule((size_t) _MemorySize, (TVmeTransferMode) _TransferMode)
{
}

TVmeLatch_SIS_36xx::~TVmeLatch_SIS_36xx()
{
}

int
TVmeLatch_SIS_36xx::Clear(int Channel)
  throw(THardwareException)
{
  PutDoubleWord(_hw_KeyReset, 0);  // reset 
  PutDoubleWord(_hw_FifoClr, 0);   // clear fifo
  PutDoubleWord(_hw_NextEna, 0);   // enable next clock logic
  return 0;
}

int                                           // returns number of bytes read
TVmeLatch_SIS_36xx::BlockRead(int Channel,    // channel number -- ignored
			      void *Data,     // destination for data
			      int MaxSize)    // maximum number of bytes
  throw(THardwareException)
{
  DoubleWord *dest = (DoubleWord*)Data;
  DoubleWord *fence= dest + MaxSize/sizeof(DoubleWord) - 1;
  DoubleWord status;

  while ((GetDoubleWord(_hw_CSR0)&_status_FifoEmpty)==0 && dest<fence)
    *(dest++)= GetDoubleWord(_hw_FifoRead);

  *(dest++)= GetDoubleWord(_hw_CSR0);
  // last double word is final status (for FIFO stat)
  
  if ( ((*dest)&_status_FifoFull)!=0 ) // fifo overflowed
    PutDoubleWord(_hw_FifoClr, 0);           // reset required to continue

  return (dest-(DoubleWord*)Data) * sizeof(DoubleWord);
}

int
TVmeLatch_SIS_36xx::WriteRegister(int Address,
				  int Data)
  throw(THardwareException)
{
  PutDoubleWord(Address, Data);
  return 0;
}
