/* module-SIS_38xx.hh */
/* Created by Glenn H-S. on 31 Aug 2000. */
/* Last updated by Glenn H-S. on 31 Aug 2000. */

#ifndef __SIS_38xx_hh__
#define __SIS_38xx_hh__

#include "RoomDefs.hh"
#include "RoomModule.hh"
#include "RoomVmeAccess.hh"


/* VmeModule implementation for SIS_38xx */
class TVmeScaler_SIS_38xx: public TVmeModule {
  public:
    TVmeScaler_SIS_38xx();
    virtual ~TVmeScaler_SIS_38xx();

    enum constant_e { N_CHANNELS=1, MAX_DATA_SIZE=(2*64*1024) };
    enum registers_e {
      _hw_CSR0      	  = 0x000,
      _hw_IdIrq     	  = 0x004,
      _hw_CopyDis   	  = 0x00C,
      _hw_FifoClr   	  = 0x020,
      _hw_VMENext   	  = 0x024,
      _hw_NextEna   	  = 0x028,
      _hw_NextDis   	  = 0x02c,
      _hw_RefPulserCh1Ena = 0x050,
      _hw_RefPulserCh1Dis = 0x054,
      _hw_KeyReset        = 0x060,
      _hw_SoftTestPulse   = 0x068,
      _hw_FifoRead        = 0x100 };
    enum bitmasks_e {
      _status_userLED         = 1<<0,
      _status_FifoTest        = 1<<1,
      _status_InpModeB0       = 1<<2,
      _status_InpModeB1       = 1<<3,
      _status_25MHzPuls       = 1<<4,
      _status_InputTest       = 1<<5,
      _status_10MHzLNEPre     = 1<<6,
      _status_LNEPreEna       = 1<<7,
      _status_FifoEmpty       = 1<<8,
      _status_FifoAlmostEmpty = 1<<9,
      _status_FifoHalfFull    = 1<<10,
      _status_FifoAlmostFull  = 1<<11,
      _status_FifoFull        = 1<<12,
      _status_RefPulserCh1    = 1<<13,
      _status_NextEna         = 1<<15,
      _status_ExtNextEna      = 1<<16,
      _status_ExtClearEna     = 1<<17,
      _status_ExtDisableEna   = 1<<18,
      _status_SoftDisableEna  = 1<<19
    };

    virtual int Clear(int Channel = -1) throw(THardwareException);

    virtual unsigned DataSize(void) throw(THardwareException)
      { return MAX_DATA_SIZE; }  // ("DataSize()" should be renamed.)
  
    virtual int NumberOfChannels(void) throw(THardwareException)
      { return N_CHANNELS; }
  
    virtual int WriteRegister(int Address, int Data) throw(THardwareException);
  
    virtual int BlockRead(int Channel, void *Data, int MaxSize)
      throw(THardwareException);
  
  private:
    enum internal_parameters_e {
      _MemorySize = 0x1000,     // vme driver can't handle < 0x1000!
      _TransferMode = tmA32D32
    };
};

#endif // __SIS_38xx_hh__
