/* module-VmeModules.cc */
/* Edited by Sanshiro E. on 26 May 1997. */
/* Last updated by Sanshiro E. on 18 June 2000. */


#include "RoomDeviceFactory.hh"
#include "RoomModule.hh"
#include "RoomVmeAccess.hh"
#include "module-VmeModules.hh"


static TVmeModuleCreator Creator1(
    "VmeMemory_General", new TVmeMemory_General()
);
static TVmeModuleCreator Creator2(
    "Rinei_RPV130", new TVmeIORegister_Rinei_RPV130()
);
static TVmeModuleCreator Creator3(
    "Hoshin_V004", new TVmeScaler_Hoshin_V004()
);


TVmeMemory_General::TVmeMemory_General(void)
:TVmeModule((size_t) VmeMemory_MapSize, (TVmeTransferMode) VmeMemory_TransferMode)
{
    _MemorySize = DefaultMemorySize;
}

TVmeMemory_General::~TVmeMemory_General()
{
}

TVmeModule* TVmeMemory_General::Clone(void)
{
    return new TVmeMemory_General();
}

int TVmeMemory_General::WriteRegister(int Address, int Data) throw(THardwareException)
{
    if (Address == regMemorySize) {
	_MemorySize = Data;
    }

    return 0;
}

int TVmeMemory_General::ReadRegister(int Address, int& Data) throw(THardwareException)
{
    if (Address == regMemorySize) {
	Data = _MemorySize;
    }

    return 0;
}

unsigned TVmeMemory_General::DataSize(void) throw(THardwareException)
{
    return _MemorySize;
}

int TVmeMemory_General::BlockRead(int Address, void *Data, int MaxSize) throw(THardwareException)
{
    DmaRead(Address, Data, _MemorySize);

    return _MemorySize;
}



TVmeIORegister_Rinei_RPV130::TVmeIORegister_Rinei_RPV130(void)
:TVmeModule((size_t) RPV130_MemorySize, (TVmeTransferMode) RPV130_TransferMode)
{
}

TVmeIORegister_Rinei_RPV130::~TVmeIORegister_Rinei_RPV130()
{
}

TVmeModule* TVmeIORegister_Rinei_RPV130::Clone(void)
{
    return new TVmeIORegister_Rinei_RPV130();
}

int TVmeIORegister_Rinei_RPV130::Initialize(int InitialState) throw(THardwareException)
{
    Clear();
    Start();

    return 0;
}

int TVmeIORegister_Rinei_RPV130::Start(int Channel) throw(THardwareException)
{
    if ((Channel == 1) || (Channel < 0)){
	*(WordPtr(CSR1)) |= (Word) Enable12;
    }
    if ((Channel == 2) || (Channel < 0)){
	*(WordPtr(CSR2)) |= (Word) Enable12;
    }
    if ((Channel == 3) || (Channel < 0)){
	*(WordPtr(CSR1)) |= (Word) Enable3;
    }

    return 0;
}

int TVmeIORegister_Rinei_RPV130::Stop(int Channel) throw(THardwareException)
{
    if ((Channel == 1) || (Channel < 0)){
	*(WordPtr(CSR1)) &= (Word) ~Enable12;
    }
    if ((Channel == 2) || (Channel < 0)){
	*(WordPtr(CSR2)) &= (Word) ~Enable12;
    }
    if ((Channel == 3) || (Channel < 0)){
	*(WordPtr(CSR1)) &= (Word) ~Enable3;
    }

    return 0;
}

int TVmeIORegister_Rinei_RPV130::IsRunning(int Channel) throw(THardwareException)
{
    int Result = 0;

    if ((Channel == 1) || (Channel < 0)) {
	Result |= GetWord(CSR1) & Enable12;	
    }
    if ((Channel == 2) || (Channel < 0)) {
	Result |= GetWord(CSR2) & Enable12;
    }
    if ((Channel == 3) || (Channel < 0)) {
	Result |= GetWord(CSR1) & Enable3;
    }

    return Result;
}

int TVmeIORegister_Rinei_RPV130::IsBusy(int Channel) throw(THardwareException)
{
    int Result = 0;

    if ((Channel == 1) || (Channel < 0)) {
	Result |= GetWord(CSR1) & Busy12;
    }
    if ((Channel == 2) || (Channel < 0)) {
	Result |= GetWord(CSR2) & Busy12;
    }
    if ((Channel == 3) || (Channel < 0)) {
	Result |= GetWord(CSR1) & Busy3;
    }

    return Result;
}

int TVmeIORegister_Rinei_RPV130::Clear(int Channel) throw(THardwareException)
{
    if ((Channel == 1) || (Channel < 0)){
	*(WordPtr(CSR1)) |= (Word) Clear12;
    }
    if ((Channel == 2) || (Channel < 0)){
	*(WordPtr(CSR2)) |= (Word) Clear12;
    }
    if ((Channel == 3) || (Channel < 0)){
	*(WordPtr(CSR1)) |= (Word) Clear3;
    }

    return 0;
}

int TVmeIORegister_Rinei_RPV130::Read(int Channel, int& Data) throw(THardwareException)
{
    switch (Channel) {
      case rtLatch1:
	Data = GetWord(Read_Latch1);
	break;
      case rtLatch2:
	Data = GetWord(Read_Latch2);
	break;
      case rtFlipFlop:
	Data = GetWord(Read_FlipFlop);
	break;
      case rtThrough:
	Data = GetWord(Read_Through);
	break;
      default:
	throw THardwareException("TVmeIORegister_Rinei_RPV130::Read()", "bad channel");
    }

    Data &= 0x00ff;
    return 1;
}

int TVmeIORegister_Rinei_RPV130::WriteRegister(int Channel, int Data) throw(THardwareException)
{
    switch (Channel) {
      case wtPulse:
	PutWord(Write_Pulse, Data);
	break;
      case wtLevel:
	PutWord(Write_Level, Data);
	break;
      default:
	throw THardwareException("TVmeIORegister_Rinei_RPV130::Write()", "bad channel");
    }

    return 1;
}



TVmeScaler_Hoshin_V004::TVmeScaler_Hoshin_V004(void)
:TVmeModule((size_t) V004_MemorySize, (TVmeTransferMode) V004_TransferMode)
{
    _DataArea = 0;
}

TVmeScaler_Hoshin_V004::~TVmeScaler_Hoshin_V004()
{
}

TVmeModule* TVmeScaler_Hoshin_V004::Clone(void)
{
    return new TVmeScaler_Hoshin_V004();
}

int TVmeScaler_Hoshin_V004::Initialize(int InitialState) throw(THardwareException)
{
    _DataArea = DoubleWordPtr();
    Stop();
    Clear();

    return 0;
}

int TVmeScaler_Hoshin_V004::Read(int Channel, int &Data) throw(THardwareException)
{
    Data = _DataArea[Channel] & DataMask;

    return 1;
}

int TVmeScaler_Hoshin_V004::Start(int Channel) throw(THardwareException)
{
    if (Channel < 0) {
	for (Channel = 0; Channel < _NumberOfChannels; Channel++) {
	    _DataArea[Channel] = StartBitPattern; 
	}
    }
    else {
	_DataArea[Channel] = StartBitPattern; 
    }

    return 0;
}

int TVmeScaler_Hoshin_V004::Stop(int Channel) throw(THardwareException)
{
    if (Channel < 0) {
	for (Channel = 0; Channel < _NumberOfChannels; Channel++) {
	    _DataArea[Channel] = StopBitPattern; 
	}
    }
    else {
	_DataArea[Channel] = StopBitPattern; 
    }
    
    return 0;
}

int TVmeScaler_Hoshin_V004::Clear(int Channel) throw(THardwareException)
{
    if (Channel < 0) {
	for (Channel = 0; Channel < _NumberOfChannels; Channel++) {
	    _DataArea[Channel] = ResetBitPattern; 
	}
    }
    else {
	_DataArea[Channel] = ResetBitPattern; 
    }
    
    return 0;
}

int TVmeScaler_Hoshin_V004::NumberOfChannels(void) throw(THardwareException)
{
    return _NumberOfChannels;
}
