/* module-VmeModules.hh */
/* Edited by Sanshiro E. on 26 May 1997. */
/* Last updated by Sanshiro E. on 18 June 2000. */

#ifndef __module_VmeModules_hh__
#define __module_VmeModules_hh__


#include "RoomModule.hh"
#include "RoomVmeAccess.hh"


/* General VME memory module */
class TVmeMemory_General: public TVmeModule {
  public:
    TVmeMemory_General(void);
    virtual ~TVmeMemory_General();
    virtual TVmeModule* Clone(void);
  public:
    virtual int WriteRegister(int Address, int Data) throw(THardwareException);
    virtual int ReadRegister(int Address, int& Data) throw(THardwareException);
    virtual unsigned DataSize(void) throw(THardwareException);
    virtual int BlockRead(int Address, void *Data, int MaxSize) throw(THardwareException);
  private:
    int _MemorySize;
  private:
    enum TVmeMemoryParameters {
	VmeMemory_MapSize = 0,
	VmeMemory_TransferMode = tmA32D32,
	DefaultMemorySize = 0x00100000 /* 1MB */
    };
    enum TVmeMemoryRegisters {
	regMemorySize = 0x0000
    };
};


/* Interrupt & I/O Register Model RPV-130 (Rinei) */
class TVmeIORegister_Rinei_RPV130: public TVmeModule {
  public:
    enum TReadTimingChannel {
	rtLatch1, rtLatch2, rtFlipFlop, rtThrough
    };
    enum TWriteTimingChannel {
	wtPulse, wtLevel
    };
  public:
    TVmeIORegister_Rinei_RPV130(void);
    virtual ~TVmeIORegister_Rinei_RPV130();
    virtual TVmeModule* Clone(void);
  public:
    virtual int Initialize(int InitialState = 0) throw(THardwareException);
    virtual int Start(int Channel = -1) throw(THardwareException);
    virtual int Stop(int Channel = -1) throw(THardwareException);
    virtual int IsRunning(int Channel = -1) throw(THardwareException);
    virtual int IsBusy(int Channel = -1) throw(THardwareException);
    virtual int Clear(int Channel = -1) throw(THardwareException);
    virtual int Read(int Channel, int& Data) throw(THardwareException);
    virtual int WriteRegister(int Channel, int Data) throw(THardwareException);
  private:
    enum TRPV130Parameters {
	RPV130_MemorySize = 0x1000,
	RPV130_TransferMode = tmA16D16
    };
    enum TRPV130Symbols {
	Read_Latch1 = 0x0000,
	Read_Latch2 = 0x0002,
	Read_FlipFlop = 0x0004,
	Read_Through = 0x0006,
	Write_Pulse = 0x0008,
	Write_Level = 0x000a,
	CSR1 = 0x000c,
	CSR2 = 0x000e,
	    Busy3 = 0x80,
	    Enable3 = 0x40,
	    Busy12 = 0x20,
	    Enable12 = 0x10,
	    Mask12 = 0x08,
	    Clear12 = 0x02,
	    Clear3 = 0x01
    };
};

/* VME 150MHz 8ch Scaler Model V004 (Hoshin) */
class TVmeScaler_Hoshin_V004: public TVmeModule {
  public:
    TVmeScaler_Hoshin_V004(void);
    virtual ~TVmeScaler_Hoshin_V004();
    virtual TVmeModule* Clone(void);
  public:
    virtual int Initialize(int InitialState = 0) throw(THardwareException);
    virtual int Read(int Channel, int& Data) throw(THardwareException);
    virtual int Start(int Channel = -1) throw(THardwareException);
    virtual int Stop(int Channel = -1) throw(THardwareException);
    virtual int Clear(int Channel= -1) throw(THardwareException);
    virtual int NumberOfChannels(void) throw(THardwareException);
  private:
    enum TV004Parameters {
	V004_MemorySize = 0x1000,
	V004_TransferMode = tmA24D32
    };
    enum TV004Symbols {
	StartBitPattern = 0x00000100UL,
	StopBitPattern  = 0x00000200UL,
	ResetBitPattern = 0x00000400UL,
	DataMask = 0x1FFFFFFFUL,
	_NumberOfChannels = 8
    };
    DoubleWord* _DataArea;
};


#endif
